@@ -3,13 +3,12 @@ package Benchmark::DKbench;
 use strict;
 use warnings;
 
-use Config;
 use Digest;
 use Digest::MD5 qw(md5_hex);
 use Encode;
 use File::Spec::Functions;
 use List::Util qw(min max sum);
-use Time::HiRes qw(CLOCK_MONOTONIC);
+use Time::HiRes;
 use Time::Piece;
 
 use Astro::Coord::Constellations 'constellation_for_eq';
@@ -30,17 +29,15 @@ use Math::MatrixReal;
 use MCE::Loop;
 use SQL::Abstract::Classic;
 use SQL::Inserter;
-use System::CPU;
 use System::Info;
 use Text::Levenshtein::Damerau::XS;
 use Text::Levenshtein::XS;
 
 use Exporter 'import';
-our @EXPORT    = qw(system_identity suite_run calc_scalability);
-our $datadir   = dist_dir("Benchmark-DKbench");
-my $mono_clock = $^O !~ /win/i || $Time::HiRes::VERSION >= 1.9764;
+our @EXPORT = qw(system_identity suite_run calc_scalability);
+our $datadir = dist_dir("Benchmark-DKbench");
 
-our $VERSION = '2.4_09';
+our $VERSION = '2.1';
 
 =head1 NAME
 
@@ -72,57 +69,25 @@ scenario.
 
 =head1 INSTALLATION
 
-See the L</"setup_dkbench"> script below for more on the installation of a couple
-of optional benchmarks and standardizing your benchmarking environment, otherwise
-here are some general guidelines for verious systems.
-
-=head2 Linux / WSL etc
-
 The only non-CPAN software required to install/run the suite is a build environment
 for the C/XS modules (C compiler, make etc.) and Perl. On the most popular Linux
 package managers you can easily set up such an environment (as root or with sudo):
 
  # Debian/Ubuntu etc
  apt-get update
- apt-get install build-essential perl cpanminus
+ apt-get -y install build-essential perl cpanminus
 
  # CentOS/Red Hat
  yum update
- yum install gcc make patch perl perl-App-cpanminus
+ yum -y install gcc make patch perl perl-App-cpanminus
 
 After that, you can use L<App::cpanminus> to install the benchmark suite (as
 root/sudo is the easiest, will install for all users):
 
  cpanm -n Benchmark::DKbench
 
-=head2 Solaris
-
-You will need to install the Oracle Solaris Studio development package to have a
-compiler environment, and to add its C<bin> directory to your PATH, before installing
-the benchmark suite.
-
-=head2 Strawberry Perl
-
-If you are on Windows, you should be using the Windows Subsystem for Linux (WSL)
-for running Perl or, if you can't (e.g. old Windows verions), cygwin instead.
-The suite should still work on Strawberry Perl, as long as you don't try to run
-tests when installing (some dependencies will not pass them). The simplest way is
-with L<App::cpanminus> (most Strawberry Perl verions have it installed):
-
- cpanm -n Benchmark::DKbench
-
-otherwise with the base CPAN shell:
-
- perl -MCPAN -e shell
-
- > notest install Benchmark::DKbench
-
-and then note that the scripts get the batch extension appended, so C<dkbench.bat>
-runs the suite (and C<setup_dkbench.bat> can assist with module versions, optional
-benchmarks etc.).
-
-Be aware that Strawberry Perl is slower, on my test system I get almost 50% slower
-performance than WSL and 30% slower than cygwin.
+See the C<setup_dkbench> script below for more on the installation of a couple of
+optional benchmarks and standardizing your benchmarking environment.
 
 =head1 SCRIPTS
 
@@ -143,22 +108,19 @@ options to control number of threads, iterations, which benchmarks to run etc:
  --multi,       -m     : Multi-threaded using all your CPU cores/threads.
  --max_threads <i>     : Override the cpu detection to specify max cpu threads.
  --iter <i>,    -i <i> : Number of suite iterations (with min/max/avg at the end).
- --stdev               : Show relative standard deviation (for iter > 1).
  --include <regex>     : Run only benchmarks that match regex.
  --exclude <regex>     : Do not run benchmarks that match regex.
  --time,        -t     : Report time (sec) instead of score.
  --quick,       -q     : Quick benchmark run (implies -t).
  --no_mce              : Do not run under MCE::Loop (implies -j 1).
- --scale <i>,   -s <i> : Scale the bench workload by x times (incompatible with -q).
  --skip_bio            : Skip BioPerl benchmarks.
+ --skip_timep          : Skip Time::Piece benchmark (see benchmark details).
  --skip_prove          : Skip Moose prove benchmark.
- --time_piece          : Run optional Time::Piece benchmark (see benchmark details).
  --bio_codons          : Run optional BioPerl Codons benchmark (does not scale well).
  --sleep <i>           : Sleep for <i> secs after each benchmark.
  --setup               : Download the Genbank data to enable the BioPerl tests.
  --datapath <path>     : Override the path where the expected benchmark data is found.
  --ver <num>           : Skip benchmarks added after the specified version.
- --help         -h     : Show basic help and exit.
 
 The default run (no options) will run all the benchmarks both single-threaded and
 multi-threaded (using all detected CPU cores/hyperthreads) and show you scores and
@@ -166,34 +128,25 @@ multi vs single threaded scalability.
 
 The scores are calibrated such that a reference CPU (Intel Xeon Platinum 8481C -
 Sapphire Rapids) would achieve a score of 1000 in a single-core benchmark run using
-the default software configuration (Linux/Perl 5.36.0 built with multiplicity and
-threads, with reference CPAN module versions). Perl built without thread support and
-multi(plicity) will be a bit faster (usually in the order of ~3-4%), while older Perl
-versions will most likely be slower. Different CPAN module versions will also impact
-scores, using C<setup_dkbench> is a way to ensure a reference environment for more
-meaningful hardware comparisons.
+the default software configuration (Linux/Perl 5.36.0 with reference CPAN modules).
 
-The multi-thread scalability calculated by the suite should approach 100% if each
-thread runs on a full core (i.e. no SMT), and the core can maintain the clock speed
-it had on the single-thread runs. Note that the overall scalability is an average
-of the benchmarks that drops non-scaling outliers (over 2*stdev less than the mean).
-
-If you want to reduce the effects of thermal throttling, which will lower the speed
-of (mainly multi-threaded) benchmarks as the CPU temperature increases, the C<sleep>
-option can help by adding cooldown time between each benchmark.
+The multi-thread scalability should approach 100% if each thread runs on a full core
+(i.e. no SMT), and the core can maintain the clock speed it had on the single-thread
+runs. Note that the overall scalability is an average of the benchmarks that drops
+non-scaling outliers (over 2*stdev less than the mean).
 
 The suite will report a Pass/Fail per benchmark. A failure may be caused if you have
 different CPAN module version installed - this is normal, and you will be warned.
 
-L<MCE::Loop> is used to run on the desired number of parallel threads, with minimal
-overhead., There is an option to disable it, which forces a single-thread run.
+The suite uses L<MCE::Loop> to run on the desired number of parallel threads, although
+there is an option to disable it, which forces a single-thread run.
 
 =head2 C<setup_dkbench>
 
 Simple installer to check/get the reference versions of CPAN modules and download
 the Genbank data file required for the BioPerl benchmarks of the DKbench suite.
 
-It assumes that you have some software already installed (see L</"INSTALLATION"> above),
+It assumes that you have some software already installed (see INSTALLATION above),
 try C<setup_dkbench --help> will give you more details.
 
  setup_dkbench [--force --sudo --test --data=s --help]
@@ -216,14 +169,14 @@ between systems and also for the benchmark Pass/Fail results to be reliable.
 
 =head1 BENCHMARKS
 
-The suite consists of 21 benchmarks, 19 will run by default. However, the
+The suite consists of 21 benchmarks, 20 will run by default. However, the
 C<BioPerl Monomers> requires the optional L<BioPerl> to be installed and Genbank
 data to be downloaded (C<dkbench --setup> can do the latter), so you will only
-see 18 benchmarks running just after a standard install. Because the overall score
-is an average, it is generally unaffected by adding or skipping a benchmark or two.
+see 19 benchmarks running just after a standard install. Because the overall score
+is an average, it is generally unaffected by skipping a benchmark or two.
 
-The optional benchmarks are enabled with the C<--time_piece> and C<--bio_codons>
-options.
+On MacOS only, the C<Time::Piece> benchmark will be skipped, unless you use C<--no_mce>
+(making it single-thread only - see why below).
 
 =over 4
 
@@ -291,10 +244,9 @@ changed (old Perl versions are faster but less strict in general).
 to 2500) are calculated using L<Text::Levenshtein::XS> and L<Text::Levenshtein::Damerau::XS>.
 
 =item * C<Time::Piece> : Creates and manipulates/converts Time::Piece objects. It
-is disabled by default because it uses the OS time libraries, so it might skew results
-if you are trying to compare CPUs on different OS platforms. It can be enabled with
-the C<--time_piece> option. For MacOS specifically, it can only be enabled if C<--no_mce>
-is specified, as it runs extremely slow when forked.
+is disabled by default on MacOS (unless C<--no_mce> is specified), as it runs
+extremely slow when forked on this platform. In general you may want to skip this
+benchmark (C<--skip_timep>) if you are comparing different OS platforms.
 
 =back
 
@@ -316,21 +268,6 @@ Prints out software/hardware configuration and returns then number of cores dete
 Runs the benchmark suite given the C<%options> and prints results. Returns a hash
 with run stats.
 
-The options of the C<dkbench> script (in their long form) are accepted, except
-C<help>, C<setup> and C<max_threads> which are exclusive to the command-line script.
-
-In addition, C<%options> may contain the key C<%extra_bench>, with a hashref value
-containing custom benchmarks in the following format:
-
- extra_bench => { bench_name => [$exp_output, $ref_time, $coderef, $quick_arg, $normal_arg] ... }
-
-Where C<bench_name> is a unique name for each benchmark and the arrayref assigned
-to it contains: The expected output (string) for the test to be considered a pass,
-the reference time in seconds for a score of 1000, a reference to the actual bench
-function, an argument (workload scaling) to pass to the function for the C<quick> 
-bench run and an argument to pass for the normal run. For more info with an example
-see the L<CUSTOM BENCHMARKS> section.
-
 =head2 C<calc_scalability>
 
  calc_scalability(\%options, \%stat_single, \%stat_multi);
@@ -338,42 +275,6 @@ see the L<CUSTOM BENCHMARKS> section.
 Given the C<%stat_single> results of a single-threaded C<suite_run> and C<%stat_multi>
 results of a multi-threaded run, will calculate and print the multi-thread scalability.
 
-=head1 CUSTOM BENCHMARKS
-
-Version 2.5 introduced the ability to add custom benchmarks to be run along any
-of the included ones of the suite. This allows you to create a suite that is more
-relevant to you, by including the actual code you will be running on the systems
-you are benchmarking. Remember, the best benchmark is your own code.
-
-Here is an example of adding a benchmark to the test suite and running it together
-with the default benchmarks:
-
-  use Benchmark::DKbench;
-  use Math::Trig qw/:great_circle :pi/;
-
-  sub great_circle {
-    my $iter = shift || 1;  # Optionally have an argument that scales the workload
-    my $dist = 0;
-    $dist +=
-      great_circle_distance(rand(pi), rand(2 * pi), rand(pi), rand(2 * pi)) -
-      great_circle_bearing(rand(pi), rand(2 * pi), rand(pi), rand(2 * pi)) +
-      great_circle_direction(rand(pi), rand(2 * pi), rand(pi), rand(2 * pi))
-      for 1 .. $iter;
-    return $dist;
-  }
-
-  my %stats = suite_run({
-      extra_bench => { 'Math::Trig' =>  # A unique name for the benchmark
-        [
-        '3144042.81433949',  # The output for your reference Perl - determines Pass/Fail
-        5.5,                 # Seconds to complete in normal mode for score = 1000
-        \&great_circle,      # Reference to bench function
-        400000,              # Argument to pass for --quick mode (if needed)
-        2000000              # Argument to pass for normal mode (if needed)
-        ]},
-    }
-  );
-
 =head1 NOTES
 
 The benchmark suite was created to compare the performance of various cloud offerings.
@@ -392,17 +293,16 @@ actual workload.
 =head2 SCORES
 
 Some sample DKbench score results from various systems for comparison (all on
-reference setup with Perl 5.36.0 thread-multi):
+reference setup with Perl 5.36.0):
 
  CPU                                     Cores/HT   Single   Multi   Scalability
  Intel i7-4750HQ @ 2.0 (MacOS)                4/8     612     2332      46.9%
- AMD Ryzen 5 PRO 4650U @ 2.1 (WSL)           6/12     905     4444      40.6%
+ AMD Ryzen 5 PRO 4650U @ 2.1 (WSL)           6/12     906     4166      38.2% 
  Apple M1 Pro @ 3.2 (MacOS)                 10/10    1283    10026      78.8%
  Apple M2 Pro @ 3.5 (MacOS)                 12/12    1415    12394      73.1%
  Ampere Altra @ 3.0 (Linux)                 48/48     708    32718      97.7%
  Intel Xeon Platinum 8481C @ 2.7 (Linux)   88/176    1000    86055      48.9%
  AMD EPYC Milan 7B13 @ 2.45 (Linux)       112/224     956   104536      49.3%
- AMD EPYC Genoa 9B14 @ 2.7 (Linux)        180/360    1197   221622      51.4%
 
 =head1 AUTHOR
 
@@ -429,10 +329,9 @@ the same terms as the Perl 5 programming language system itself.
 =cut
 
 sub benchmark_list {
-    my $extra_bench = shift || {};
     return {               # idx : 0 = result, 1 = ref time, 2 = func, 3 = quick test, 4 = normal test, 5 = ver
         'Astro'             => ['e71c7ae08f16fe26aea7cfdb72785873', 5.674, \&bench_astro, 20000, 80000],
-        'BioPerl Codons'    => ['97c443c099886ca60e99f7ab9df689b5', 8.752, \&bench_bioperl_codons, 3, 5],
+        'BioPerl Codons'    => ['97c443c099886ca60e99f7ab9df689b5', 8.752, \&bench_bioperl_codons, 3, 5, 1],
         'BioPerl Monomers'  => ['d29ed0a5c205c803c112be1338d1f060', 5.241, \&bench_bioperl_mono, 6, 20],
         'Crypt::JWT'        => ['d41d8cd98f00b204e9800998ecf8427e', 6.451, \&bench_jwt, 250, 900],
         'CSS::Inliner'      => ['82c1b6de9ca0500a48f8a8df0998df3c', 4.603, \&bench_css, 2, 5],
@@ -452,23 +351,27 @@ sub benchmark_list {
         'Regex/Subst utf8'  => ['857eb4e63a4d174ca4a16fe678f7626f', 5.703, \&bench_regex_utf8, 3, 10],
         'Text::Levenshtein' => ['2948a300ed9131fa0ce82bb5eabb8ded', 5.539, \&bench_textlevenshtein, 7, 25, 2.1],
         'Time::Piece'       => ['2d4b149fe7f873a27109fc376d69211b', 5.907, \&bench_timepiece, 75_000, 275_000],
-        %$extra_bench
     };
 }
 
 sub system_identity {
-    my ($physical, $cores, $ncpu) = System::CPU::get_cpu;
-    $ncpu ||= 1;
+    my ($physical, $cores, $ncpu) =
+        $^O =~ /bsd|darwin|dragonfly/ ? bsd_cpu() : linux_cpu();
+    $ncpu ||= MCE::Util::get_ncpu() || 1;
     local $^O = 'linux' if $^O =~ /android/;
     my $info  = System::Info->sysinfo_hash;
     my $osn   = $info->{distro} || $info->{os} || $^O;
-    my $model = System::CPU::get_name || '';
-    my $arch  = System::CPU::get_arch || '';
+    my $model = $info->{cpu};
+    my $arch  = $info->{cpu_type} || '';
     $arch = " ($arch)" if $arch;
+    if ($^O =~ /darwin/ && $model =~ /^Mac|nknown/) {
+        my $brand = `sysctl -a |grep brand`;
+        $model = $1 if $brand =~ /brand_string: (.*)/;
+    }
+    $model =~ s/\s+/ /g;
+    $model =~ s/\(R\)//g;
     print "--------------- Software ---------------\nDKbench v$VERSION\n";
-    printf "Perl $^V (%sthreads, %smulti)\n",
-        $Config{usethreads}      ? '' : 'no ',
-        $Config{usemultiplicity} ? '' : 'no ',;
+    print "Perl $^V\n";
     print "OS: $osn\n--------------- Hardware ---------------\n";
     print "CPU type: $model$arch\n";
     print "CPUs: $ncpu";
@@ -482,12 +385,39 @@ sub system_identity {
     return $ncpu;
 };
 
+sub bsd_cpu {
+    chomp( my $cpus = `sysctl -n hw.ncpu 2>/dev/null` );
+    return unless $cpus;
+    chomp( my $cores = `sysctl -n hw.physicalcpu 2>/dev/null` );
+    $cores ||= $cpus;
+    return (undef, $cores, $cpus);
+}
+
+sub linux_cpu {
+    my (@physical, @cores, $phys, $cpus);
+    if ( -f '/proc/cpuinfo' && open my $fh, '<', '/proc/cpuinfo' ) {
+        while (<$fh>) {
+            $cpus++ if /^processor\s*:/;
+            push @physical, $1 if /^physical id\s*:\s*(\d+)/;
+            push @cores, $1 if /^cpu cores\s*:\s*(\d+)/;
+        }
+        return undef, $cores[0], $cpus if !@physical && @cores;
+        @cores = (0) unless @cores;
+        my %hash;
+        $hash{$physical[$_]} = $_ < scalar(@cores) ? $cores[$_] : $cores[0]
+            for 0 .. $#physical;
+        my $phys  = keys %hash || undef;
+        my $cores = sum(values %hash) || $cpus;
+        return $phys, $cores, $cpus;
+    }
+    return;
+}
+
 sub suite_run {
     my $opt = shift;
     $datadir = $opt->{datapath} if $opt->{datapath};
     $opt->{threads} //= 1;
-    $opt->{scale} //= 1;
-    $opt->{iter} ||= 1;
+    $opt->{repeat} //= 1;
     $opt->{f} = $opt->{time} ? '%.3f' : '%5.0f';
     my %stats = (threads => $opt->{threads});
 
@@ -508,9 +438,9 @@ sub suite_run {
 
 sub calc_scalability {
     my ($opt, $stats1, $stats2) = @_;
-    my $benchmarks = benchmark_list($opt->{extra_bench});
-    my $threads    = $stats2->{threads} / $stats1->{threads};
-    my $display    = $opt->{time} ? 'times' : 'scores';
+    my $benchmarks = benchmark_list();
+    my $threads = $stats2->{threads}/$stats1->{threads};
+    my $display = $opt->{time} ? 'times' : 'scores';
     $opt->{f} = $opt->{time} ? '%.3f' : '%5.0f';
     my (@perf, @scal);
     print "Multi thread Scalability:\n".pad_to("Benchmark",24).pad_to("Multi perf xSingle",24).pad_to("Multi scalability %",24);
@@ -528,14 +458,7 @@ sub calc_scalability {
     print (("-"x40)."\n");
     my $avg1 = min_max_avg($stats1->{total}->{$display});
     my $avg2 = min_max_avg($stats2->{total}->{$display});
-    print "DKbench summary ($cnt benchmark";
-    print "s" if $cnt > 1;
-    print " x$opt->{scale} scale" if $opt->{scale} && $opt->{scale} > 1;
-    print ", $opt->{iter} iterations" if $opt->{iter} && $opt->{iter} > 1;
-    print ", $stats2->{threads} thread";
-    print "s" if $stats2->{threads} > 1;
-    print "):\n";
-    $opt->{f} .= "s" if $opt->{time};
+    print "DKbench summary ($cnt benchmarks, $stats2->{threads} threads):\n";
     print pad_to("Single:").sprintf($opt->{f}, $avg1)."\n";
     print pad_to("Multi:").sprintf($opt->{f}, $avg2)."\n";
     my @newperf = Benchmark::DKbench::drop_outliers(\@perf, -1);
@@ -548,8 +471,8 @@ sub calc_scalability {
 
 sub run_iteration {
     my ($opt, $stats) = @_;
-    my $benchmarks    = benchmark_list($opt->{extra_bench});
-    my $title         = $opt->{time} ? 'Time (sec)' : 'Score';
+    my $benchmarks = benchmark_list();
+    my $title = $opt->{time} ? 'Time (sec)' : 'Score';
     print pad_to("Benchmark").pad_to($title);
     print "Pass/Fail" unless $opt->{time};
     print "\n";
@@ -558,7 +481,7 @@ sub run_iteration {
         next if $opt->{skip_bio} && $bench =~ /Monomers/;
         next if $opt->{skip_prove} && $bench =~ /prove/;
         next if !$opt->{bio_codons} && $bench =~ /Codons/;
-        next if !$opt->{time_piece} && $bench =~ /Time::Piece/;
+        next if $opt->{skip_timep} && $bench =~ /Time::Piece/;
         next if $opt->{ver} && $benchmarks->{$bench}->[5] && $opt->{ver} < $benchmarks->{$bench}->[5];
         next if $opt->{exclude} && $bench =~ /$opt->{exclude}/;
         next if $opt->{include} && $bench !~ /$opt->{include}/;
@@ -600,7 +523,7 @@ sub mce_bench_run {
             MCE->gather([$time, $res]);
         }
     }
-    (1 .. $opt->{threads} * $opt->{scale});
+    (1 .. $opt->{threads} * $opt->{repeat});
 
     my ($res, $time) = ('Pass', 0);
     foreach (@stats) {
@@ -608,16 +531,16 @@ sub mce_bench_run {
         $res = $_->[1] if $_->[1] ne 'Pass';
     }
 
-    return $time/($opt->{threads}*$opt->{scale} || 1), $res;
+    return $time/($opt->{threads}*$opt->{repeat} || 1), $res;
 }
 
 sub bench_run {
     my ($benchmark, $srand) = @_;
     $srand //= 1;
     srand($srand); # For repeatability
-    my $t0   = _get_time();
+    my $t0   = Time::HiRes::time();
     my $out  = $benchmark->[2]->($benchmark->[3]);
-    my $time = sprintf("%.3f", _get_time()-$t0);
+    my $time = sprintf("%.3f", Time::HiRes::time()-$t0);
     my $r    = $out eq $benchmark->[0] ? 'Pass' : "Fail ($out)";
     return $time, $r;
 }
@@ -635,7 +558,7 @@ sub bench_astro {
 
 sub bench_bioperl_codons {
     my $skip = shift;
-    my $iter = shift || 1;
+    my $iter = shift;
     my $d    = Digest->new("MD5");
     my $file = catfile($datadir, "gbbct5.seq");
     foreach (1..$iter) {
@@ -1100,11 +1023,10 @@ sub bench_timepiece {
 
 sub total_stats {
     my ($opt, $stats) = @_;
-    my $benchmarks = benchmark_list($opt->{extra_bench});
-    my $display    = $opt->{time} ? 'times' : 'scores';
-    my $title      = $opt->{time} ? 'Time (sec)' : 'Score';
-    print "Aggregates ($opt->{iter} iterations):\n".pad_to("Benchmark",24).pad_to("Avg $title").pad_to("Min $title").pad_to("Max $title");
-    print pad_to("stdev %") if $opt->{stdev};
+    my $benchmarks = benchmark_list();
+    my $display = $opt->{time} ? 'times' : 'scores';
+    my $title   = $opt->{time} ? 'Time (sec)' : 'Score';
+    print "Aggregates:\n".pad_to("Benchmark",24).pad_to("Avg $title").pad_to("Min $title").pad_to("Max $title");
     print pad_to("Pass %") unless $opt->{time};
     print "\n";
     foreach my $bench (sort keys %$benchmarks) {
@@ -1125,13 +1047,7 @@ sub calc_stats {
     my $arr = shift;
     my $pad = shift;
     my ($min, $max, $avg) = min_max_avg($arr);
-    my $str = join '', map {pad_to(sprintf($opt->{f}, $_), $pad)} ($avg,$min,$max);
-    if ($opt->{stdev} && $avg) {
-        my $stdev = avg_stdev($arr);
-        $stdev *= 100/$avg;
-        $str .= pad_to(sprintf("%0.2f%%", $stdev), $pad);
-    }
-    return $avg, $str;
+    return $avg, join '', map {pad_to(sprintf($opt->{f}, $_), $pad)} ($avg,$min,$max);
 }
 
 sub min_max_avg {
@@ -1333,10 +1249,6 @@ sub _decode_jwt2 {
     return $payload;
 }
 
-sub _get_time {
-    return $mono_clock ? Time::HiRes::clock_gettime(CLOCK_MONOTONIC) : Time::HiRes::time();
-}
-
 # Helper package for Moose benchmark
 
 {
